--Addon created by Erick Goldner

local transform_lap
local qualy_player_lap
local race_player_positions
local race_fastest_lap


-- Function to transform lap in milliseconts do MM:SS:mmm format      
local function transform_lap( lap )

	local totalseconds = math.floor(lap / 1000)
	local milliseconds = lap % 1000
  	local seconds = totalseconds % 60
  	local minutes = math.floor(totalseconds / 60)
  	minutes = minutes % 60
  	return string.format("%02d:%02d:%03d", minutes, seconds, milliseconds)  

end

-- Participant lap finished on Qualify. Prints position, Participant Name and Lap Time from those who achieve 1, 2, 3 position after lap completed.
local function qualy_player_lap( participant )

	local qualy_att_player_name = participant.attributes.Name
	local qualy_att_fastest_lap = participant.attributes.FastestLapTime
	local qualy_att_race_position = participant.attributes.RacePosition
	local qualy_att_fastest_lap_formatted = 0

	if qualy_att_race_position == 1 or qualy_att_race_position == 2 or qualy_att_race_position == 3 then
		qualy_att_fastest_lap_formatted = transform_lap(qualy_att_fastest_lap)
		SendChatToAll( "Pos.: " .. qualy_att_race_position .. ", " .. qualy_att_player_name .. ", Time: " .. qualy_att_fastest_lap_formatted )
	end
end

-- Participant lap finished on Race. Prints info from the first 3 positions
local function race_player_positions( participant )
	
	local race_player_name = participant.attributes.Name
    	local race_player_current_position = participant.attributes.RacePosition
	local race_player_last_lap_time = participant.attributes.LastLapTime
	local race_player_current_lap = participant.attributes.CurrentLap
	local race_player_last_lap_time_formatted = 0 
	race_player_last_lap_time_formatted = transform_lap(race_player_last_lap_time)

	if race_player_current_position == 1 then
			SendChatToAll( "Lap " .. race_player_current_lap .. " completed!")
			SendChatToAll( "Pos.: " .. race_player_current_position .. ", " .. race_player_name .. ", Time: " .. race_player_last_lap_time_formatted )
	end

	if race_player_current_position == 2 or race_player_current_position == 3 then
			SendChatToAll( "Pos.: " .. race_player_current_position .. ", " .. race_player_name .. ", Time: " .. race_player_last_lap_time_formatted )
	end
	
end

-- Participant lap finished on Race. Prints the Fastest Lap on the Race.
local function race_fastest_lap( participant)
	
	local race_fastest_lap_time = 0
	local race_fastest_lap_name
	
	local race_att_player_name = participant.attributes.Name
	local race_att_fastest_lap = participant.attributes.FastestLapTime
	local race_att_current_lap = participant.attributes.CurrentLap

		for RefId,partici in pairs( session.participants ) do
			local part_name = partici.attributes.Name
			local part_fastest_lap = partici.attributes.FastestLapTime
			if (part_fastest_lap ~= 0) and (race_fastest_lap_time == 0) then
				race_fastest_lap_time = part_fastest_lap
				race_fastest_lap_name = part_name
			elseif (part_fastest_lap ~= 0) and (part_fastest_lap < race_att_fastest_lap) then
				race_fastest_lap_time = part_fastest_lap
				race_fastest_lap_name = part_name
			end
		end
		
		if race_att_fastest_lap <= race_fastest_lap_time then
			local race_event_lap_time_formatted = transform_lap(race_att_fastest_lap)
			SendChatToAll( race_att_player_name .. " has set the Fastest Lap (" .. race_att_current_lap .. "): " .. race_event_lap_time_formatted)	
		end
end

-- Main addon callback
local function addon_callback( callback, ... )

	-- Handle events.
	if callback == Callback.ParticipantAttributesChanged then
		local participantid, attrlist = ...
		local participant = session.participants[ participantid ]
		local attrset = table.list_to_set( attrlist )
			-- Completed Valid Lap on Qualy Event
			if (session.attributes.SessionStage == "Qualifying") and (attrset.CurrentLap) and (attrset.FastestLapTime) then
				qualy_player_lap( participant )
			end
			-- Completed Lap on Race and it's the Fastest Lap
			if (session.attributes.SessionStage == "Race1") and (participant.attributes.CurrentLap > 1)  and (attrset.FastestLapTime) then
				race_fastest_lap( participant )
			end
			-- Completed Lap on Race and it's the first 3 positions
			if (session.attributes.SessionStage == "Race1") and (attrset.CurrentLap) then
				if participant.attributes.CurrentLap > 1 then
					race_player_positions( participant )
				end
			end
	end
end

-- Callbacks
RegisterCallback( addon_callback )
EnableCallback( Callback.Tick )
EnableCallback( Callback.EventLogged )
EnableCallback( Callback.ParticipantAttributesChanged)